# frozen_string_literal: true

# Crea la tabla de permalinks que lleva el registro de todas las URLs
# posibles de un artículo.
#
# A través de los permalinks podemos:
#
# * Mantener un listado estable de todas las rutas de un artículo y
# llevar el listado de alternate-permalinks, para nunca romper un
# permalink.
#
# * Relacionar artículos con sus visitas.
#
# * Llevar un registro de todos los archivos estáticos (imágenes, audio,
# video, documentos) y sus descargas.
class CreatePermalinks < ActiveRecord::Migration[6.1]
  def up
    create_table :permalinks, id: false do |t|
      t.primary_key :id, :uuid, default: 'public.gen_random_uuid()'
      t.belongs_to :site, null: false, index: true
      t.string :permalinkeable_type, null: false
      t.string :permalinkeable_id, null: false
      t.string :url, null: false
      t.timestamps
    end

    add_index :permalinks, %i[permalinkeable_type permalinkeable_id]
    add_index :permalinks, %i[site_id url], unique: true

    # XXX: Esta migración va a ignorar las URLs duplicadas.
    Site.find_each do |site|
      next unless site.jekyll

      site.docs.each do |post|
        post.to_index.tap do |indexed_post|
          indexed_post.permalinks.build url: post.document.url, site: site
        end.save
      end
    end
  end

  def down
    drop_table :permalinks
  end
end
